#ifndef __CTMM_LIB_H
#define __CTMM_LIB_H

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#ifdef AUDIT
#include <libaudit.h>
#endif

#define CTMM_DIR			"/ctmm/"
//#define CTMMM_HOOK_DIR			"hooks/"

#define CTMM_ENABLE 			CTMM_DIR "enable"
#define CTMM_ACCESS 			CTMM_DIR "access"
#define CTMM_AUDIT			CTMM_DIR "log"
#define CTMM_POLICY			CTMM_DIR "policy"
#define CTMM_CONFIG			CTMM_DIR "config"
#define CTMM_RECOVERY			CTMM_DIR "recovery"
#define CTMM_FIND_POLICY		CTMM_DIR "find_policy"
#define CTMM_SET_POLICY			CTMM_DIR "set_policy"
#define CTMM_SUPER			CTMM_DIR "super"

#define CTMM_BOOT_HOOK			CTMM_DIR "hooks/boot_hook"
#define CTMM_EXEC_HOOK			CTMM_DIR "hooks/exec_hook"
#define CTMM_LIB_HOOK			CTMM_DIR "hooks/lib_hook"
#define CTMM_FILE_HOOK			CTMM_DIR "hooks/file_hook"
#define CTMM_MODLOAD_HOOK		CTMM_DIR "hooks/mod_load_hook"
#define CTMM_MODDEL_HOOK		CTMM_DIR "hooks/mod_del_hook"

#define CTMM_INIT_CONFIG		"/etc/ctmm/ctmm.cfg"

#define GRUB_CMDLINE                 "/proc/cmdline"

#define CONFIG_LINE_LEN 		9

#define BUF_SIZE 			1024

#define MEASURE_SIZE 			33

#define KEY_SIZE			256

#define CTMM_WHITELIST_DB_FILE "/etc/ctmm/whitelist.sqlite"

#define PERMISSION_SIZE         10

#define SIZE                    1024

#define ERR_LOG(format, arg...) fprintf(stderr,"%s: " format, MY_NAME , ## arg)

enum ctmm_hook_status { HOOK_LIST_OFF = 0,  HOOK_LIST_ON , HOOK_LIST_ALL};
 
enum
{
	LABEL_EXE = 0x00000001,
	LABEL_REL = 0x00000002,
	LABEL_LIB = 0x00000004,
	LABEL_SCR = 0x00000008
};

enum ctmm_file_type {TYPE_REG=1, //regular
					TYPE_SCR=2,  //script
					TYPE_LIB=3,  //library
					TYPE_EXE=4,  //exec
					TYPE_MOD=5,  //module
					};    
enum name_type {FULL_NAME = 1, SHORT_NAME};

/*
*ܣװûļ
*ûϢļ,<Ϊ>
*ֵ<0>ɹ<-1>ʧ
*/
int load_config(char *config_file);


/*
*ܣװûϢ
*ûļ<Ϊ>
*ֵ<0>ɹ<-1>ʧ
*/
int load_policy(char *policy_file);
int load_super_policy(char *policy_file);

int load_policy_sysroot(char *policy_file, char *sysroot);

//add by fu.tang
void get_file_permission(char *filepath, char *filemode, int size);

void get_rpm_by_file(char *filename, char *package, int size);

int get_file_type(const char *filename, int *file_type);
//add by fu.tang

/*
*ܣû
*
*	label_dir: ҪɨɰĿ¼Ϊգ
	<ΪĬɨ/bin/,/sbin/,/usr/bin,/usr/sbin/,/usr/local/bin/,/etc,6Ŀ¼>
* 	outpu_file: ɵİļƣΪա
*	appendʾɨɵİǷҪ׷ӵļ<1>׷ӣ<0>׷
*	flags: Ҫɨļ
*ֵ<0>ɹ<-1>ʧ
*/
int init_wlt_list(char *label_dir,bool append, int flags, char *output_file);
int init_wlt_file(char *filename, bool append, int flags,char *output_file);

/*
*ܣļļ
*
*	file_name: Ҫӵļеļ,Ϊ.
*	output_fileɵĶļƣΪ
*	appendʾļǷ׷ӵɵĶļУ<1>׷ӣ<0>׷
*ֵ<0>ɹ<-1>ʧ
*/
int add_config(char *file_name, char *output_file, bool append);

/*
*ܣӶļɾָļ
*
*	file_name: Ҫɾļ,Ϊ.
*	output_fileļ,Ϊ
*ֵ<0>ɹ<-1>ʧ
*/
int del_config(char *file_name, char *config_file);



/*
*ܣȡctmmģϢ
*
*	file: ҪȡļϢ
*ֵ---
*ע
*1ctmmģ鿪״̬.
*	file = /sys/kernel/security/ctmm/enable
*	ֵ<1>:״̬<0>:ر״̬<-1>:
*2ctmmģʽ.
*	file = /sys/kernel/security/ctmm/mode
*	ֵ<1>:ģʽ<0>:ǿģʽ<-1>:
*3ctmmģʽ.
*	file = /sys/kernel/security/ctmm/audit
*	ֵ<-1>:
*			<0>:ƹر
*			<1>:оܾ
*			<2>:
*			<3>:в
*4ctmm̶̬״̬.
*	file = /sys/kernel/security/ctmm/process_measure/enable
*	ֵ<1>:̶̬<0>:رգ<-1>:
*/
int get_value(char * file);

/*
*ܣctmmģϢ
*
*	file: ҪҪļ
*ֵ---
*ע
*1ctmmģ鿪״̬.
*	
*		file/sys/kernel/security/ctmm/enable
*		value<1>:<0>ر	
*	ֵ<0>:ɹ<-1>:
*2ctmmģʽ.
*	
*		file/sys/kernel/security/ctmm/mode
*		value: <1>:ģʽ<0>:ǿģʽ
*	ֵ<0>:ɹ<-1>:
*3ctmmģʽ.
*	
*		file/sys/kernel/security/ctmm/audit
*		value<0>:ƹر <1>:оܾ <2>: <3>:в
*	ֵ<0>:ɹ<-1>:
*4ctmm̶̬״̬.
*	
*		file/sys/kernel/security/ctmm/process_measure/enable
*		value<1>:̶̬<0>:ر
*	ֵ<0>:ɹ<-1>:
*/
int set_value(char *file, int value);

/*
*ܣָļļhashֵԼļݵĶֵ
*
*	file: Ҫļ
*	name_hashļhashֵ
*	file_type: ļ
*	digestļݵĶֵ
*ֵ---С0<0>
*ע
*/
int get_file_measure(char *file_name, unsigned int *name_hash, int* file_type,  char (digest)[MEASURE_SIZE]);

/*
 * ܣָļļɱݹ鵵ļ
 * 
 *	cpio_fileմɺõıݹ鵵ļƲΪ
 *	backup_configҪݵļбļΪգ
 *	appendݺõĹ鵵ļѾʱǷҪµӵԭеĹ鵵ļ
 *ֵ---С0<0>
 */
int ctmm_backup(char *cpio_file, char *backup_config, bool append);

/*
 * ܣɨָĿ¼ļб
 * 
 *	cpio_fileմɺõıݹ鵵ļƲΪ
 *  (˹鵵ļô׷ӱݣ򴴽¹鵵ļ)
 *	path_dirҪݵĿ¼ļ
 *ֵ---С0<0>
 */
int ctmm_backup_path(char *cpio_file, char *path_dir);

/*
 * ܣƶļָʱ״̬
 * 
 *	cpio_fileմɺõıݹ鵵ļΪ
 *	file_nameȴָļ
 * ֵ---С0<0>
 */
int ctmm_recovery(char *cpio_file, char *file_name);

/*
 * ܣںctmmnetlinkϢļ
 * 
 *	
 * ֵ---С0<0>
 */
int open_ctmm_sock();

/*
 * ܣںctmmϢ֪ͨӦļѻָ
 * 
 *	sockfdopen_ctmm_sock()򿪲صsocketļ
 *	messageѾָ״̬ļ
 *	lenѾָ״̬ļƵĳ
 * ֵ---С0<0>
 */
int send_ctmm_message(int sockfd, char *message, int len);

/*
 * ܣںctmm͵ûռϢļʧܣ
 * 
 *	sockfdopen_ctmm_sock()򿪲صsocketļ
 *	message洢ں˶ʧܵļַ
 *	lenַС
 * ֵ---С0<0>
 */
int receive_ctmm_message(int sockfd, char *message, int len);

/*
 * ܣرںctmmnetlinksocketļ
 * 
 *	sockfdopen_ctmm_sock()򿪲صsocketļ
 * ֵ---С0<0>
 */
int close_ctmm_sock(int sockfd);


int sm3_hash_file(char *file, char *output, int size);
int sm3_hash_elf(char *file, char *output, int size);


int get_whitelist_policy(char *filename, int kind);

void get_file_counts(char *dir, int flags);

int need_label(char *filename, int flags, int *file_type, bool *hash_lib);
#endif
